import { isListing } from "$lib/Listing";
import { isLoginData } from "$lib/Login";
import { readListingByQuery } from "$lib/server/mongo";
import { getRequestBody } from "$lib/server/requestTools";
import {
	badRequestResponse,
	notFoundResponse,
	singleResponse,
	unauthorizedResponse
} from "$lib/server/responseBodies";
import type { RequestHandler } from "@sveltejs/kit";
import { JWT_SECRET } from "$env/static/private";
import { compare } from "bcrypt";
import jwt from "jsonwebtoken";

export const POST: RequestHandler = async ({ locals }): Promise<Response> => {
	try {
		const { user } = locals;

		if (user.kind !== "Basic") {
			return unauthorizedResponse();
		}

		const { username, password } = user.payload;
		const listing = await readListingByQuery(
			"logins",
			{
				"data.username": username
			},
			(target) => isListing(target, isLoginData)
		);

		if (!listing) {
			return notFoundResponse();
		}

		if (await compare(password, listing.data.password)) {
			const token = await jwt.sign(
				{ sub: listing.id, username, role: listing.data.role },
				JWT_SECRET,
				{
					expiresIn: "1d"
				}
			);

			return singleResponse(token);
		}

		return badRequestResponse("wrong password");
	} catch (err) {
		return badRequestResponse("username and password are required");
	}
};
